@@ -47,7 +47,7 @@ radv_is_vertex_buffer_format_supported(VkFormat format)
    if (first_non_void < 0)
       return false;
 
-   const struct util_format_description *desc = vk_format_description(format);
+   const struct util_format_description *desc = radv_format_description(format);
    return ac_translate_buffer_dataformat(desc, first_non_void) != V_008F0C_BUF_DATA_FORMAT_INVALID;
 }
 
@@ -80,7 +80,7 @@ radv_translate_tex_numformat(const struct util_format_description *desc, int fir
 static bool
 radv_is_sampler_format_supported(const struct radv_physical_device *pdev, VkFormat format, bool *linear_sampling)
 {
-   const struct util_format_description *desc = vk_format_description(format);
+   const struct util_format_description *desc = radv_format_description(format);
    uint32_t num_format;
    if (format == VK_FORMAT_UNDEFINED || format == VK_FORMAT_R64_UINT || format == VK_FORMAT_R64_SINT)
       return false;
@@ -94,7 +94,8 @@ radv_is_sampler_format_supported(const struct radv_physical_device *pdev, VkForm
       *linear_sampling = true;
    else
       *linear_sampling = false;
-   return radv_translate_tex_dataformat(pdev, vk_format_description(format),
+
+   return radv_translate_tex_dataformat(pdev, radv_format_description(format),
                                         vk_format_get_first_non_void_channel(format)) != ~0U;
 }
 
@@ -109,7 +110,7 @@ bool
 radv_is_storage_image_format_supported(const struct radv_physical_device *pdev, VkFormat format)
 {
    const struct radv_instance *instance = radv_physical_device_instance(pdev);
-   const struct util_format_description *desc = vk_format_description(format);
+   const struct util_format_description *desc = radv_format_description(format);
    unsigned data_format, num_format;
    if (format == VK_FORMAT_UNDEFINED)
       return false;
@@ -199,7 +200,7 @@ radv_is_buffer_dataformat_supported(const struct util_format_description *desc,
 bool
 radv_is_buffer_format_supported(VkFormat format, bool *scaled)
 {
-   const struct util_format_description *desc = vk_format_description(format);
+   const struct util_format_description *desc = radv_format_description(format);
    unsigned num_format;
 
    if (format == VK_FORMAT_UNDEFINED)
@@ -222,7 +223,7 @@ radv_is_buffer_format_supported(VkFormat format, bool *scaled)
 static bool
 radv_is_colorbuffer_format_blendable(const struct radv_physical_device *pdev, VkFormat format)
 {
-   const struct util_format_description *desc = vk_format_description(format);
+   const struct util_format_description *desc = radv_format_description(format);
    const uint32_t color_format = ac_get_cb_format(pdev->info.gfx_level, desc->format);
    const uint32_t color_num_format = ac_get_cb_number_type(desc->format);
 
@@ -238,7 +239,7 @@ radv_is_colorbuffer_format_blendable(const struct radv_physical_device *pdev, Vk
 bool
 radv_is_colorbuffer_format_supported(const struct radv_physical_device *pdev, VkFormat format)
 {
-   const struct util_format_description *desc = vk_format_description(format);
+   const struct util_format_description *desc = radv_format_description(format);
    return ac_is_colorbuffer_format_supported(pdev->info.gfx_level, desc->format);
 }
 
@@ -274,7 +275,7 @@ radv_physical_device_get_format_properties(struct radv_physical_device *pdev, Vk
                                            VkFormatProperties3 *out_properties)
 {
    VkFormatFeatureFlags2 linear = 0, tiled = 0, buffer = 0;
-   const struct util_format_description *desc = vk_format_description(format);
+   const struct util_format_description *desc = radv_format_description(format);
    bool scaled = false;
    /* TODO: implement some software emulation of SUBSAMPLED formats. */
    if (desc->format == PIPE_FORMAT_NONE || desc->layout == UTIL_FORMAT_LAYOUT_SUBSAMPLED) {
@@ -501,7 +502,7 @@ radv_physical_device_get_format_properties(struct radv_physical_device *pdev, Vk
 bool
 radv_format_pack_clear_color(VkFormat format, uint32_t clear_vals[2], VkClearColorValue *value)
 {
-   const struct util_format_description *desc = vk_format_description(format);
+   const struct util_format_description *desc = radv_format_description(format);
 
    if (format == VK_FORMAT_B10G11R11_UFLOAT_PACK32) {
       clear_vals[0] = float3_to_r11g11b10f(value->float32);
@@ -674,8 +675,14 @@ radv_list_drm_format_modifiers(struct radv_physical_device *pdev, VkFormat forma
       if (!features)
          continue;
 
-      unsigned planes =
-         vk_format_get_plane_count(format) + ac_modifier_has_dcc(mods[i]) + ac_modifier_has_dcc_retile(mods[i]);
+      unsigned planes = vk_format_get_plane_count(format);
+
+      if (pdev->info.gfx_level < GFX12) {
+         /* DCC is transparent to the userspace driver on GFX12 so it doesn't
+          * need additional planes.
+          */
+         planes += ac_modifier_has_dcc(mods[i]) + ac_modifier_has_dcc_retile(mods[i]);
+      }
 
       vk_outarray_append_typed(VkDrmFormatModifierPropertiesEXT, &out, out_props)
       {
@@ -725,8 +732,14 @@ radv_list_drm_format_modifiers_2(struct radv_physical_device *pdev, VkFormat for
       if (!features)
          continue;
 
-      unsigned planes =
-         vk_format_get_plane_count(format) + ac_modifier_has_dcc(mods[i]) + ac_modifier_has_dcc_retile(mods[i]);
+      unsigned planes = vk_format_get_plane_count(format);
+
+      if (pdev->info.gfx_level < GFX12) {
+         /* DCC is transparent to the userspace driver on GFX12 so it doesn't
+          * need additional planes.
+          */
+         planes += ac_modifier_has_dcc(mods[i]) + ac_modifier_has_dcc_retile(mods[i]);
+      }
 
       vk_outarray_append_typed(VkDrmFormatModifierProperties2EXT, &out, out_props)
       {
@@ -855,7 +868,7 @@ radv_get_image_format_properties(struct radv_physical_device *pdev, const VkPhys
    uint32_t maxMipLevels;
    uint32_t maxArraySize;
    VkSampleCountFlags sampleCounts = VK_SAMPLE_COUNT_1_BIT;
-   const struct util_format_description *desc = vk_format_description(format);
+   const struct util_format_description *desc = radv_format_description(format);
    enum amd_gfx_level gfx_level = pdev->info.gfx_level;
    VkImageTiling tiling = info->tiling;
    const VkPhysicalDeviceImageDrmFormatModifierInfoEXT *mod_info =
@@ -1219,7 +1232,7 @@ radv_GetPhysicalDeviceImageFormatProperties2(VkPhysicalDevice physicalDevice,
           *    vkGetPhysicalDeviceImageFormatProperties2 returns
           *    VK_ERROR_FORMAT_NOT_SUPPORTED.
           */
-         result = vk_errorf(pdev, VK_ERROR_FORMAT_NOT_SUPPORTED, "unsupported VkExternalMemoryTypeFlagBitsKHR 0x%x",
+         result = vk_errorf(pdev, VK_ERROR_FORMAT_NOT_SUPPORTED, "unsupported VkExternalMemoryHandleTypeFlagBits 0x%x",
                             external_info->handleType);
          goto fail;
       }
@@ -1500,8 +1513,8 @@ radv_dcc_formats_compatible(enum amd_gfx_level gfx_level, VkFormat format1, VkFo
    if (format1 == format2)
       return true;
 
-   desc1 = vk_format_description(format1);
-   desc2 = vk_format_description(format2);
+   desc1 = radv_format_description(format1);
+   desc2 = radv_format_description(format2);
 
    if (desc1->nr_channels != desc2->nr_channels)
       return false;
