@@ -329,8 +329,14 @@ lp_build_fill_mattrs(std::vector<std::string> &MAttrs)
     * which allows us to enable/disable code generation based
     * on the results of cpuid on these architectures.
     */
-   llvm::StringMap<bool> features;
-   llvm::sys::getHostCPUFeatures(features);
+   #if LLVM_VERSION_MAJOR >= 19
+      /* llvm-19+ returns StringMap from getHostCPUFeatures.
+      */
+      auto features = llvm::sys::getHostCPUFeatures();
+   #else
+      llvm::StringMap<bool> features;
+      llvm::sys::getHostCPUFeatures(features);
+   #endif
 
    for (llvm::StringMapIterator<bool> f = features.begin();
         f != features.end();
@@ -377,6 +383,7 @@ lp_build_fill_mattrs(std::vector<std::string> &MAttrs)
    MAttrs.push_back(util_get_cpu_caps()->has_avx512bw ? "+avx512bw"  : "-avx512bw");
    MAttrs.push_back(util_get_cpu_caps()->has_avx512dq ? "+avx512dq"  : "-avx512dq");
    MAttrs.push_back(util_get_cpu_caps()->has_avx512vl ? "+avx512vl"  : "-avx512vl");
+   MAttrs.push_back(util_get_cpu_caps()->has_avx512vbmi ? "+avx512vbmi"  : "-avx512vbmi");
 #endif
 #if DETECT_ARCH_ARM
    if (!util_get_cpu_caps()->has_neon) {
@@ -419,8 +426,6 @@ lp_build_fill_mattrs(std::vector<std::string> &MAttrs)
 
 #if DETECT_ARCH_LOONGARCH64 == 1
    /*
-    * TODO: Implement util_get_cpu_caps()
-    *
     * No FPU-less LoongArch64 systems are ever shipped yet, and LP64D is
     * the default ABI, so FPU is enabled here.
     *
@@ -429,9 +434,16 @@ lp_build_fill_mattrs(std::vector<std::string> &MAttrs)
     * https://github.com/loongson/la-softdev-convention/releases/download/v0.1/la-softdev-convention.pdf
     */
    MAttrs = {"+f","+d"};
-#if LLVM_VERSION_MAJOR == 17
-   /* LLVM 17's LSX support is incomplete, so explicitly mask it */
+#if LLVM_VERSION_MAJOR >= 18
+   MAttrs.push_back(util_get_cpu_caps()->has_lsx ? "+lsx" : "-lsx");
+   MAttrs.push_back(util_get_cpu_caps()->has_lasx ? "+lasx" : "-lasx");
+#else
+   /*
+    * LLVM 17's LSX support is incomplete, and LLVM 16 isn't supported
+    * LSX and LASX. So explicitly mask it.
+    */
    MAttrs.push_back("-lsx");
+   MAttrs.push_back("-lasx");
 #endif
 #endif
 }
