@@ -7,14 +7,16 @@ use mesa_rust_gen::*;
 use mesa_rust_util::serialize::*;
 use mesa_rust_util::string::*;
 
+use std::ffi::CStr;
 use std::ffi::CString;
 use std::fmt::Debug;
+use std::ops::Not;
 use std::os::raw::c_char;
 use std::os::raw::c_void;
 use std::ptr;
 use std::slice;
 
-const INPUT_STR: *const c_char = b"input.cl\0" as *const u8 as *const c_char;
+const INPUT_STR: &CStr = c"input.cl";
 
 pub enum SpecConstant {
     None,
@@ -31,8 +33,8 @@ unsafe impl Sync for SPIRVBin {}
 
 #[derive(PartialEq, Eq, Hash, Clone)]
 pub struct SPIRVKernelArg {
-    pub name: String,
-    pub type_name: String,
+    pub name: CString,
+    pub type_name: CString,
     pub access_qualifier: clc_kernel_arg_access_qualifier,
     pub address_qualifier: clc_kernel_arg_address_qualifier,
     pub type_qualifier: clc_kernel_arg_type_qualifier,
@@ -43,7 +45,7 @@ pub struct CLCHeader<'a> {
     pub source: &'a CString,
 }
 
-impl<'a> Debug for CLCHeader<'a> {
+impl Debug for CLCHeader<'_> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let name = self.name.to_string_lossy();
         let source = self.source.to_string_lossy();
@@ -79,7 +81,7 @@ unsafe extern "C" fn spirv_to_nir_msg_callback(
 
 fn create_clc_logger(msgs: &mut Vec<String>) -> clc_logger {
     clc_logger {
-        priv_: msgs as *mut Vec<String> as *mut c_void,
+        priv_: ptr::from_mut(msgs).cast(),
         error: Some(spirv_msg_callback),
         warning: Some(spirv_msg_callback),
     }
@@ -92,7 +94,7 @@ impl SPIRVBin {
         headers: &[CLCHeader],
         cache: &Option<DiskCache>,
         features: clc_optional_features,
-        spirv_extensions: &[CString],
+        spirv_extensions: &[&CStr],
         address_bits: u32,
     ) -> (Option<Self>, String) {
         let mut hash_key = None;
@@ -140,7 +142,7 @@ impl SPIRVBin {
             headers: c_headers.as_ptr(),
             num_headers: c_headers.len() as u32,
             source: clc_named_value {
-                name: INPUT_STR,
+                name: INPUT_STR.as_ptr(),
                 value: source.as_ptr(),
             },
             args: c_args.as_ptr(),
@@ -155,7 +157,7 @@ impl SPIRVBin {
         let logger = create_clc_logger(&mut msgs);
         let mut out = clc_binary::default();
 
-        let res = unsafe { clc_compile_c_to_spirv(&args, &logger, &mut out) };
+        let res = unsafe { clc_compile_c_to_spirv(&args, &logger, &mut out, ptr::null_mut()) };
 
         let res = if res {
             let spirv = SPIRVBin {
@@ -175,12 +177,12 @@ impl SPIRVBin {
             None
         };
 
-        (res, msgs.join("\n"))
+        (res, msgs.join(""))
     }
 
     // TODO cache linking, parsing is around 25% of link time
     pub fn link(spirvs: &[&SPIRVBin], library: bool) -> (Option<Self>, String) {
-        let bins: Vec<_> = spirvs.iter().map(|s| &s.spirv as *const _).collect();
+        let bins: Vec<_> = spirvs.iter().map(|s| ptr::from_ref(&s.spirv)).collect();
 
         let linker_args = clc_linker_args {
             in_objs: bins.as_ptr(),
@@ -206,7 +208,7 @@ impl SPIRVBin {
             spirv: out,
             info: info,
         });
-        (res, msgs.join("\n"))
+        (res, msgs.join(""))
     }
 
     pub fn validate(&self, options: &clc_validator_options) -> (bool, String) {
@@ -214,7 +216,7 @@ impl SPIRVBin {
         let logger = create_clc_logger(&mut msgs);
         let res = unsafe { clc_validate_spirv(&self.spirv, &logger, options) };
 
-        (res, msgs.join("\n"))
+        (res, msgs.join(""))
     }
 
     pub fn clone_on_validate(&self, options: &clc_validator_options) -> (Option<Self>, String) {
@@ -224,12 +226,14 @@ impl SPIRVBin {
 
     fn kernel_infos(&self) -> &[clc_kernel_info] {
         match self.info {
-            None => &[],
-            Some(info) => unsafe { slice::from_raw_parts(info.kernels, info.num_kernels as usize) },
+            Some(info) if info.num_kernels > 0 => unsafe {
+                slice::from_raw_parts(info.kernels, info.num_kernels as usize)
+            },
+            _ => &[],
         }
     }
 
-    fn kernel_info(&self, name: &str) -> Option<&clc_kernel_info> {
+    pub fn kernel_info(&self, name: &str) -> Option<&clc_kernel_info> {
         self.kernel_infos()
             .iter()
             .find(|i| c_string_to_string(i.name) == name)
@@ -243,54 +247,25 @@ impl SPIRVBin {
             .collect()
     }
 
-    pub fn vec_type_hint(&self, name: &str) -> Option<String> {
-        self.kernel_info(name)
-            .filter(|info| [1, 2, 3, 4, 8, 16].contains(&info.vec_hint_size))
-            .map(|info| {
-                let cltype = match info.vec_hint_type {
-                    clc_vec_hint_type::CLC_VEC_HINT_TYPE_CHAR => "uchar",
-                    clc_vec_hint_type::CLC_VEC_HINT_TYPE_SHORT => "ushort",
-                    clc_vec_hint_type::CLC_VEC_HINT_TYPE_INT => "uint",
-                    clc_vec_hint_type::CLC_VEC_HINT_TYPE_LONG => "ulong",
-                    clc_vec_hint_type::CLC_VEC_HINT_TYPE_HALF => "half",
-                    clc_vec_hint_type::CLC_VEC_HINT_TYPE_FLOAT => "float",
-                    clc_vec_hint_type::CLC_VEC_HINT_TYPE_DOUBLE => "double",
-                };
-
-                format!("vec_type_hint({}{})", cltype, info.vec_hint_size)
-            })
-    }
-
-    pub fn local_size(&self, name: &str) -> Option<String> {
-        self.kernel_info(name)
-            .filter(|info| info.local_size != [0; 3])
-            .map(|info| {
-                format!(
-                    "reqd_work_group_size({},{},{})",
-                    info.local_size[0], info.local_size[1], info.local_size[2]
-                )
-            })
-    }
-
-    pub fn local_size_hint(&self, name: &str) -> Option<String> {
-        self.kernel_info(name)
-            .filter(|info| info.local_size_hint != [0; 3])
-            .map(|info| {
-                format!(
-                    "work_group_size_hint({},{},{})",
-                    info.local_size_hint[0], info.local_size_hint[1], info.local_size_hint[2]
-                )
-            })
-    }
-
     pub fn args(&self, name: &str) -> Vec<SPIRVKernelArg> {
         match self.kernel_info(name) {
             Some(info) if info.num_args > 0 => {
                 unsafe { slice::from_raw_parts(info.args, info.num_args) }
                     .iter()
                     .map(|a| SPIRVKernelArg {
-                        name: c_string_to_string(a.name),
-                        type_name: c_string_to_string(a.type_name),
+                        // SAFETY: we have a valid C string pointer here
+                        name: a
+                            .name
+                            .is_null()
+                            .not()
+                            .then(|| unsafe { CStr::from_ptr(a.name) }.to_owned())
+                            .unwrap_or_default(),
+                        type_name: a
+                            .type_name
+                            .is_null()
+                            .not()
+                            .then(|| unsafe { CStr::from_ptr(a.type_name) }.to_owned())
+                            .unwrap_or_default(),
                         access_qualifier: clc_kernel_arg_access_qualifier(a.access_qualifier),
                         address_qualifier: a.address_qualifier,
                         type_qualifier: clc_kernel_arg_type_qualifier(a.type_qualifier),
@@ -301,10 +276,36 @@ impl SPIRVBin {
         }
     }
 
+    fn get_spirv_capabilities() -> spirv_capabilities {
+        spirv_capabilities {
+            Addresses: true,
+            Float16: true,
+            Float16Buffer: true,
+            Float64: true,
+            GenericPointer: true,
+            Groups: true,
+            GroupNonUniformShuffle: true,
+            GroupNonUniformShuffleRelative: true,
+            Int8: true,
+            Int16: true,
+            Int64: true,
+            Kernel: true,
+            ImageBasic: true,
+            ImageReadWrite: true,
+            Linkage: true,
+            LiteralSampler: true,
+            SampledBuffer: true,
+            Sampled1D: true,
+            Vector16: true,
+            ..Default::default()
+        }
+    }
+
     fn get_spirv_options(
         library: bool,
         clc_shader: *const nir_shader,
         address_bits: u32,
+        caps: &spirv_capabilities,
         log: Option<&mut Vec<String>>,
     ) -> spirv_to_nir_options {
         let global_addr_format;
@@ -320,7 +321,7 @@ impl SPIRVBin {
 
         let debug = log.map(|log| spirv_to_nir_options__bindgen_ty_1 {
             func: Some(spirv_to_nir_msg_callback),
-            private_data: (log as *mut Vec<String>).cast(),
+            private_data: ptr::from_mut(log).cast(),
         });
 
         spirv_to_nir_options {
@@ -330,25 +331,8 @@ impl SPIRVBin {
             float_controls_execution_mode: float_controls::FLOAT_CONTROLS_DENORM_FLUSH_TO_ZERO_FP32
                 as u32,
 
-            caps: spirv_supported_capabilities {
-                address: true,
-                float16: true,
-                float64: true,
-                generic_pointers: true,
-                groups: true,
-                subgroup_shuffle: true,
-                int8: true,
-                int16: true,
-                int64: true,
-                kernel: true,
-                kernel_image: true,
-                kernel_image_read_write: true,
-                linkage: true,
-                literal_sampler: true,
-                printf: true,
-                ..Default::default()
-            },
-
+            printf: true,
+            capabilities: caps,
             constant_addr_format: global_addr_format,
             global_addr_format: global_addr_format,
             shared_addr_format: offset_addr_format,
@@ -369,7 +353,9 @@ impl SPIRVBin {
         log: Option<&mut Vec<String>>,
     ) -> Option<NirShader> {
         let c_entry = CString::new(entry_point.as_bytes()).unwrap();
-        let spirv_options = Self::get_spirv_options(false, libclc.get_nir(), address_bits, log);
+        let spirv_caps = Self::get_spirv_capabilities();
+        let spirv_options =
+            Self::get_spirv_options(false, libclc.get_nir(), address_bits, &spirv_caps, log);
 
         let nir = unsafe {
             spirv_to_nir(
@@ -390,7 +376,9 @@ impl SPIRVBin {
     pub fn get_lib_clc(screen: &PipeScreen) -> Option<NirShader> {
         let nir_options = screen.nir_shader_compiler_options(pipe_shader_type::PIPE_SHADER_COMPUTE);
         let address_bits = screen.compute_param(pipe_compute_cap::PIPE_COMPUTE_CAP_ADDRESS_BITS);
-        let spirv_options = Self::get_spirv_options(true, ptr::null(), address_bits, None);
+        let spirv_caps = Self::get_spirv_capabilities();
+        let spirv_options =
+            Self::get_spirv_options(false, ptr::null(), address_bits, &spirv_caps, None);
         let shader_cache = DiskCacheBorrowed::as_ptr(&screen.shader_cache());
 
         NirShader::new(unsafe {
@@ -434,6 +422,10 @@ impl SPIRVBin {
 
     pub fn spec_constant(&self, spec_id: u32) -> Option<clc_spec_constant_type> {
         let info = self.info?;
+        if info.num_spec_constants == 0 {
+            return None;
+        }
+
         let spec_constants =
             unsafe { slice::from_raw_parts(info.spec_constants, info.num_spec_constants as usize) };
 
@@ -468,46 +460,45 @@ impl Drop for SPIRVBin {
 }
 
 impl SPIRVKernelArg {
-    pub fn serialize(&self) -> Vec<u8> {
-        let mut res = Vec::new();
+    pub fn serialize(&self, blob: &mut blob) {
+        unsafe {
+            blob_write_uint32(blob, self.access_qualifier.0);
+            blob_write_uint32(blob, self.type_qualifier.0);
 
-        let name_arr = self.name.as_bytes();
-        let type_name_arr = self.type_name.as_bytes();
+            blob_write_string(blob, self.name.as_ptr());
+            blob_write_string(blob, self.type_name.as_ptr());
 
-        res.extend_from_slice(&name_arr.len().to_ne_bytes());
-        res.extend_from_slice(name_arr);
-        res.extend_from_slice(&type_name_arr.len().to_ne_bytes());
-        res.extend_from_slice(type_name_arr);
-        res.extend_from_slice(&u32::to_ne_bytes(self.access_qualifier.0));
-        res.extend_from_slice(&u32::to_ne_bytes(self.type_qualifier.0));
-        res.push(self.address_qualifier as u8);
-
-        res
+            blob_write_uint8(blob, self.address_qualifier as u8);
+        }
     }
 
-    pub fn deserialize(bin: &mut &[u8]) -> Option<Self> {
-        let name_len = read_ne_usize(bin);
-        let name = read_string(bin, name_len)?;
-        let type_len = read_ne_usize(bin);
-        let type_name = read_string(bin, type_len)?;
-        let access_qualifier = read_ne_u32(bin);
-        let type_qualifier = read_ne_u32(bin);
+    pub fn deserialize(blob: &mut blob_reader) -> Option<Self> {
+        unsafe {
+            let access_qualifier = blob_read_uint32(blob);
+            let type_qualifier = blob_read_uint32(blob);
 
-        let address_qualifier = match read_ne_u8(bin) {
-            0 => clc_kernel_arg_address_qualifier::CLC_KERNEL_ARG_ADDRESS_PRIVATE,
-            1 => clc_kernel_arg_address_qualifier::CLC_KERNEL_ARG_ADDRESS_CONSTANT,
-            2 => clc_kernel_arg_address_qualifier::CLC_KERNEL_ARG_ADDRESS_LOCAL,
-            3 => clc_kernel_arg_address_qualifier::CLC_KERNEL_ARG_ADDRESS_GLOBAL,
-            _ => return None,
-        };
+            let name = blob_read_string(blob);
+            let type_name = blob_read_string(blob);
 
-        Some(Self {
-            name: name,
-            type_name: type_name,
-            access_qualifier: clc_kernel_arg_access_qualifier(access_qualifier),
-            address_qualifier: address_qualifier,
-            type_qualifier: clc_kernel_arg_type_qualifier(type_qualifier),
-        })
+            let address_qualifier = match blob_read_uint8(blob) {
+                0 => clc_kernel_arg_address_qualifier::CLC_KERNEL_ARG_ADDRESS_PRIVATE,
+                1 => clc_kernel_arg_address_qualifier::CLC_KERNEL_ARG_ADDRESS_CONSTANT,
+                2 => clc_kernel_arg_address_qualifier::CLC_KERNEL_ARG_ADDRESS_LOCAL,
+                3 => clc_kernel_arg_address_qualifier::CLC_KERNEL_ARG_ADDRESS_GLOBAL,
+                _ => return None,
+            };
+
+            // check overrun to ensure nothing went wrong
+            blob.overrun.not().then(|| Self {
+                // SAFETY: blob_read_string checks for a valid nul character already and sets the
+                //         blob to overrun state if none was found.
+                name: CStr::from_ptr(name).to_owned(),
+                type_name: CStr::from_ptr(type_name).to_owned(),
+                access_qualifier: clc_kernel_arg_access_qualifier(access_qualifier),
+                address_qualifier: address_qualifier,
+                type_qualifier: clc_kernel_arg_type_qualifier(type_qualifier),
+            })
+        }
     }
 }
 
@@ -532,3 +523,59 @@ impl CLCSpecConstantType for clc_spec_constant_type {
         }
     }
 }
+
+pub trait SpirvKernelInfo {
+    fn vec_type_hint(&self) -> Option<String>;
+    fn local_size(&self) -> Option<String>;
+    fn local_size_hint(&self) -> Option<String>;
+
+    fn attribute_str(&self) -> String {
+        let attributes_strings = [
+            self.vec_type_hint(),
+            self.local_size(),
+            self.local_size_hint(),
+        ];
+
+        let attributes_strings: Vec<_> = attributes_strings.into_iter().flatten().collect();
+        attributes_strings.join(",")
+    }
+}
+
+impl SpirvKernelInfo for clc_kernel_info {
+    fn vec_type_hint(&self) -> Option<String> {
+        if ![1, 2, 3, 4, 8, 16].contains(&self.vec_hint_size) {
+            return None;
+        }
+        let cltype = match self.vec_hint_type {
+            clc_vec_hint_type::CLC_VEC_HINT_TYPE_CHAR => "uchar",
+            clc_vec_hint_type::CLC_VEC_HINT_TYPE_SHORT => "ushort",
+            clc_vec_hint_type::CLC_VEC_HINT_TYPE_INT => "uint",
+            clc_vec_hint_type::CLC_VEC_HINT_TYPE_LONG => "ulong",
+            clc_vec_hint_type::CLC_VEC_HINT_TYPE_HALF => "half",
+            clc_vec_hint_type::CLC_VEC_HINT_TYPE_FLOAT => "float",
+            clc_vec_hint_type::CLC_VEC_HINT_TYPE_DOUBLE => "double",
+        };
+
+        Some(format!("vec_type_hint({}{})", cltype, self.vec_hint_size))
+    }
+
+    fn local_size(&self) -> Option<String> {
+        if self.local_size == [0; 3] {
+            return None;
+        }
+        Some(format!(
+            "reqd_work_group_size({},{},{})",
+            self.local_size[0], self.local_size[1], self.local_size[2]
+        ))
+    }
+
+    fn local_size_hint(&self) -> Option<String> {
+        if self.local_size_hint == [0; 3] {
+            return None;
+        }
+        Some(format!(
+            "work_group_size_hint({},{},{})",
+            self.local_size_hint[0], self.local_size_hint[1], self.local_size_hint[2]
+        ))
+    }
+}
